/* *********************************************************************** 
** From SQL Server Magazine - September 2002 
** Use this script to understand File and Filegroup Backup/Restore

** NOTE: This script is best viewed in Courier New 12pt
*********************************************************************** */

/* *********************************************************************** 
** This script will create another copy of the Pubs database named
** PubsTest. PubsTest will be created in the default location of SQL
** SQL Server on drive c:\. Change C:\Program Files\Microsoft SQL 
** Server\mssql\data\ if you did NOT install SQL Server in the default 
** directory and/or path. Make sure you MODIFY this script before 
** running it. 
** 
** In addition to copying Pubs to the new PubsTest database, this script 
** will add 5 files TOTALLING 30 MB of additional space. In all this 
** script requires roughly 40 MB of space.
** 
** Also, although this script will run in it's entirety, this script is 
** NOT intended to be executed in one run. My recommendation is to work 
** your way through this script reading and reviewing the comments
** for better understanding.
** 
** As you execute portions of this script, you will modify data in a this
** new PubsTest database and create different types of backups. Once
** backed up you will use msdb queries to determine the likely order 
** of file & filegroup backups as well as the log backups needed to restore. 
** 
** At the very end there are a couple of questions asked. To restore 
** the database successfully in these scenarios check out the answers script!
** 
** That should really help you test yourself!
** 
** Have fun,
** kt
*********************************************************************** */

USE pubs
go

BACKUP DATABASE [pubs] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' WITH INIT
go

RESTORE DATABASE [PubsTest] 
	FROM  DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH  FILE = 1,  NOUNLOAD ,  STATS = 10,  RECOVERY ,  
MOVE N'pubs_log' TO N'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\PubsTestLog.ldf',  
MOVE N'pubs' TO N'C:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTest.mdf'
go

ALTER DATABASE [PubsTest] 
	SET RECOVERY FULL
go

USE [PubsTest] 
go

-- Increase the size of the Transaction Log for later activity
ALTER DATABASE [PubsTest] 
MODIFY FILE
		(NAME = N'pubs_log', 
		SIZE = 7, FILEGROWTH = 10%)
go

-- Add Additional Files and a FileGroup.
ALTER DATABASE [PubsTest] ADD FILEGROUP [RWFG] 
go

ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestRWFile1', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile1.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [RWFG]
go
ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestRWFile2', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile2.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [RWFG]
go
ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestRWFile3', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile3.NDF' , 
		SIZE = 6, FILEGROWTH = 10%) 
		TO FILEGROUP [RWFG]
go
ALTER DATABASE [PubsTest] ADD FILEGROUP [ROFG] 
go

ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestROFile1', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestROFile1.NDF' , 
		SIZE = 5, FILEGROWTH = 10%) 
		TO FILEGROUP [ROFG]
go
ALTER DATABASE [PubsTest] 
	ADD FILE
		(NAME = N'PubsTestROFile2', 
		FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestROFile2.NDF' , 
		SIZE = 5, FILEGROWTH = 10%) 
		TO FILEGROUP [ROFG]
go

USE PubsTest
go
sp_helpfile -- To Review the current number of and size of the database files.
go
ALTER DATABASE PubsTest
	MODIFY FILEGROUP [RWFG] DEFAULT
-- This allows you to place objects on the Filegroup WITHOUT specifying it every time!
go

CREATE TABLE dbo.DemoTable
(
	col1	int	 identity(100,10),
	col2	datetime default current_timestamp,
	col3	datetime default getdate(),
	col4	char(30) default suser_name(),
	col5	char(30) default user_name(),
	col6	char(80) default 'Now is the time for all good men to come to the aid of their country.'
) -- ON [RWFG] -- this is NOT required if RWFG is the DEFAULT FileGroup.
go

SELECT count(*) FROM dbo.DemoTable	-- 0 Rows but the table does exist.
go
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

SELECT count(*) FROM dbo.DemoTable	-- 2000 Rows
go

DBCC SQLPERF(logspace)
-- PubsTest should show about 30% unless you have modified any of the settings...
go

-- The next section will create a table specifically on the ROFG. 
-- Once the data is populated the ROFG will be set to READONLY. 
-- For completeness the syntax is shown to set it back to READWRITE.

CREATE TABLE dbo.DemoTableForROFG
(
	col1	int	 identity(100,10),
	col2	datetime default current_timestamp,
	col3	datetime default getdate(),
	col4	char(30) default suser_name(),
	col5	char(30) default user_name(),
	col6	char(80) default 'Now is the time for all good men to come to the aid of their country.'
) ON [ROFG] -- this is required because RWFG is the DEFAULT FileGroup.
go

SELECT count(*) FROM dbo.DemoTableForROFG	-- 0 Rows but the table does exist.
go
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

SELECT count(*) FROM dbo.DemoTableForROFG	-- 2000 Rows
go

/* *********************************************************************** 
** IMPORTANT NOTE: Due to a bug in SQL Server 2000 RTM, sp1 and sp2 - I 
** DO NOT recommend using the MODIFY FILEGROUP option for ALTER DATABASE
** to set the filegroup attributes to READONLY. In fact, I recommend
** that you create a filegroup that will effectively be used as READONLY
** but I do not recommend that you set this property. If set (and
** feel free to use this script to uncomment this out and do some tests)
** you will be unable to restore transaction log backups when recovering
** from an isolated failure. A complete database restore must be 
** initiated. 
** 
** In order to ensure that you can recover quickly - in the case of 
** isolated disk failure, do not set any filegroups to readonly. Even if
** the failure were in another filegroup the restore will still fail.

** To test this you can uncomment out the alter to set the filegroup to 
** READONLY and TRY to use the RestorePartialDBFileFilegroupStrategy.sql 
** to recovery. You will find that the transaction logs cannot be applied.
*********************************************************************** */

-- ALTER DATABASE PubsTest
-- 	MODIFY FILEGROUP [ROFG] READONLY
-- go

-- ALTER DATABASE PubsTest
-- 		MODIFY FILEGROUP [fg] READWRITE
-- go

DBCC SQLPERF(logspace)
-- PubsTest should show about 50% unless you have modified any of the settings...
go

-- Now that the database is ready we can begin the File and Filegroup
-- strategy for backups. Typically for a database this size most DBAs 
-- would perform FULL and TLOG backups however larger databases would 
-- add periodic Differential Backups. VLDBs would break it down into 
-- FILE/FILEGROUP and TLOG Backups and even larger databases would use 
-- add FILE/FILEGROUP with Differential Backups.

-- So - to demostrate the point of how to use the most complex scenario
-- FILE/FILEGROUP with Differential Backups - we will create use the
-- PubsTest database which has 7 total files.

-- It is important to remember that you must first setup FULL backups
-- at either the File or the Filegroup level and that ALL files must be 
-- represented. The idea is that you can create file backups over the 
-- period of a couple of days - to create the entire framework of your
-- database. These could be staggered over a few days (when the files 
-- themselves are so large that you are unable to back them up at the 
-- same time).

sp_helpfile -- to review File/Filegroup Names
go
-- FileName			#	Physical Location														Filegroup	Size	Max Size	Growth	Usage
-- pubs				1	C:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTest.mdf 	      	PRIMARY		1600 KB	Unlimited	10%		data only	
-- pubs_log    		2	C:\Program Files\Microsoft SQL Server\MSSQL\DATA\PubsTestLog.ldf      	NULL		5120 KB	Unlimited	10%		log only
-- PubsTestRWFile1	3	c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile1.NDF  	RWFG		6144 KB	Unlimited	10%		data only	
-- PubsTestRWFile2	4	c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile2.NDF  	RWFG		6144 KB	Unlimited	10%		data only	
-- PubsTestRWFile3  5	c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestRWFile3.NDF  	RWFG		6144 KB	Unlimited	10%		data only	
-- PubsTestROFile1	6	c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestROFile1.NDF  	ROFG		6144 KB	Unlimited	10%		data only	
-- PubsTestROFile2	7	c:\Program Files\Microsoft SQL Server\MSSQL\data\PubsTestROFile2.NDF  	ROFG		6144 KB	Unlimited	10%		data only	

-- Clear all PubsTest Backup History First!
USE msdb
go

PRINT ' '
PRINT 'CLEARING ALL Backup History First'
EXEC sp_delete_database_backuphistory 'PubsTest'
go

USE PubsTest
go

PRINT ' '
PRINT 'Step 1 - File Backup, File = pubs (Primary File)'
BACKUP DATABASE [PubsTest] FILE = 'pubs'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, File = pubs', INIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
PRINT ' '
PRINT 'Step 2 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup OR the 3 Files Individually.
-- Backing up the filegroup is easier but the total size of the filegroup
-- may be too large in some environments as well. The individual file 
-- backups are another option but these are commented out.

PRINT ' '
PRINT 'Step 3 - FileGroup Backup, FileGroup = RWFG'
BACKUP DATABASE [PubsTest] FILEGROUP = 'RWFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup = RWFG', NOINIT
go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestRWFile1'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestRWFile1', NOINIT
-- go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestRWFile2'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestRWFile2', NOINIT
-- go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestRWFile3'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestRWFile3', NOINIT

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
PRINT ' '
PRINT 'Step 4 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the ReadOnly FileGroup (ROFG) OR the 2 Files Individually.
-- Backing up the filegroup is easier but the total size of the filegroup
-- may be too large in some environments as well. The individual file 
-- backups are another option but these are commented out.

PRINT ' '
PRINT 'Step 5 - FileGroup Backup, FileGroup = ROFG'
BACKUP DATABASE [PubsTest] FILEGROUP = 'ROFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup = ROFG', NOINIT
go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestROFile1'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestROFile1', NOINIT
-- go
-- BACKUP DATABASE [PubsTest] FILE = 'PubsTestROFile2'
-- 	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
--	WITH NAME = 'PubsTest Backup, File = PubsTestROFile2', NOINIT
-- go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
PRINT ' '
PRINT 'Step 6 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup with Differential
PRINT ' '
PRINT 'Step 7 - FileGroup DIFFERENTIAL Backup, FileGroup = RWFG'
BACKUP DATABASE [PubsTest] FILEGROUP = 'RWFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup DIFFERENTIAL = RWFG', DIFFERENTIAL, NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Create an object that modifies the Primary Filegroup
CREATE PROCEDURE dbo.GetDataFromDemoTable
AS
SELECT * FROM dbo.DemoTable
WHERE Col1 % 19 = 0
go

-- Backup the Log
PRINT ' '
PRINT 'Step 8 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

PRINT ' '
PRINT 'Step 9 - File Backup, File = pubs (Primary File)'
BACKUP DATABASE [PubsTest] FILE = 'pubs'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, File = pubs', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
PRINT ' '
PRINT 'Step 10 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the FileGroup with Differential
PRINT ' '
PRINT 'Step 11 - FileGroup DIFFERENTIAL Backup, FileGroup = RWFG'
BACKUP DATABASE [PubsTest] FILEGROUP = 'RWFG'
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FileGroup DIFFERENTIAL = RWFG', DIFFERENTIAL, NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Backup the Log
PRINT ' '
PRINT 'Step 12 - Transaction Log Backup'
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, Transaction Log', NOINIT
go

-- Change some more data
SET NOCOUNT ON
DECLARE @RowCounter	int
SELECT @RowCounter = 1
WHILE @RowCounter < 2001
	BEGIN
		INSERT dbo.DemoTable DEFAULT VALUES
		SELECT @RowCounter = @RowCounter + 1
	END
go

-- Review the backup history in MSDB
SELECT Backup_Start_Date, 
	[Name],
	[Description],
	First_LSN, 
	Last_LSN, 
	Backup_Finish_Date, 
	* -- OR Just *
FROM msdb.dbo.backupset AS s
    JOIN msdb.dbo.backupmediafamily AS m
        ON s.media_set_id = m.media_set_id
WHERE database_name = 'PubsTest'
ORDER BY 1 ASC
