/* Use this script to understand File and Filegroup Backup/Restore. It
   builds on the scenario created by the "FileFilegroupStrategiesCaseStudy.sql" 
   and helps you to review the process to restore the whole database. */

-- Begin by reviewing the backup history in MSDB, if it exists.
SELECT *
FROM msdb.dbo.backupset AS s
    JOIN msdb.dbo.backupmediafamily AS m
        ON s.media_set_id = m.media_set_id
WHERE database_name = 'PubsTest'
ORDER BY 1 ASC

-- Make sure that you know where and what type each backup is; good naming conventions
-- really help here. You could get the same information from the backup devices.

LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
-- What are the last FULL file/filegroup backups (by position) type = 4?

LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 1 -- Position number on a multifile backup
LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 2 -- Position number on a multifile backup
LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 10 -- Position number on a multifile backup
-- With a full filegroup backup, all files are listed from LOAD FILELISTONLY.

-- Find the last differentials:
LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
-- What are the last file/filegroup DIFFERENTIAL backups (by position) type = 6?

-- Which log series do you restore? Check the LSN.
LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
--What are the last log series by LSN (by position) type = 2?

-- A complete recover to another location requires all File/FileGroup backups to be
-- restored first, then Differentials, then finally all logs with the minimum LSN. 

-- First, make sure you have the tail of the log:
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FINAL Transaction Log (Tail up-to-minute)', NOINIT,
	NO_TRUNCATE
go

-- In the main article, review what you have already backed up.
-- To set up the database, we need the "pubs" file (backup number 9),
-- then the RWFG FileGroup (backup number 3), and then the ROFG 
-- FileGroup (backup number 5). Once the framework of the database
-- has been recreated, we need the latest differentials of everything.
-- This means the latest diff of the RWFG filegroup (backup number 11).
-- Now you're ready to apply the transaction logs in order to roll
-- forward this database to the point in time of the failure.

RESTORE DATABASE NewPubsTest
	FILE = 'pubs'
	-- OR FILEGROUP = 'Primary'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 9, -- Backup number by position
NORECOVERY
go

RESTORE DATABASE NewPubsTest
	FILEGROUP = 'RWFG'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 3, -- Backup number by position
NORECOVERY
go

RESTORE DATABASE NewPubsTest
	FILEGROUP = 'ROFG'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestROFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestROFile1.ndf',
	MOVE 'PubsTestROFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestROFile2.ndf',
FILE = 5, -- Backup number by position
NORECOVERY
go

RESTORE DATABASE NewPubsTest
	FILEGROUP = 'RWFG'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 11, -- Backup number by position
NORECOVERY
go

-- The database is set up, but the data hasn't been rolled forward. You
-- need to apply the correct sequence of transaction logs. To determine
-- the minimum effective LSN, look at the MIN LSN for each of the last 
-- Full File or last Differentials you restored. The lowest would be 
-- backup number 5 (not backup number 3 - because 7 updated this one with 
-- the differential changes). You need to review the min LSNs and find the 
-- first transaction log backup that has a MIN LSN that is less than the
-- MIN LSN of backup 5. In this case, you want the transaction log
-- backup in backup 6. However, you might have a scenario where the first
-- log to apply is earlier than the backup. Always review the MIN LSNs
-- to make sure that you have the correct sequence.

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 6, -- Backup number by position
NORECOVERY
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 8, -- Backup number by position
NORECOVERY
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 10, -- Backup number by position
NORECOVERY 
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 12, -- Backup number by position
NORECOVERY 
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestRWFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile1.ndf',
	MOVE 'PubsTestRWFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile2.ndf',
	MOVE 'PubsTestRWFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestRWFile3.ndf',
FILE = 13, -- Backup number by position
RECOVERY 
go
SELECT count(*) FROM PubsTest.dbo.DemoTable 	-- You should have 28,000 rows in PubsTest.
SELECT count(*) FROM NewPubsTest.dbo.DemoTable 	-- You should have 28,000 rows in NewPubsTest.

-- If you accidentally left the last log with NORECOVERY, you could run this statement
-- at the end to bring the NewPubsTest database online:
-- RESTORE DATABASE NewPubsTest WITH RECOVERY

-- Also, because this was a complete restore, you could have used STOPAT to return to an earlier 
-- point in time. 
